// **************************************************************
//     Filename: wavTrigger.cpp
// Date Created: 2/23/2014
//
//     Comments: Robertsonics WAV Trigger serial control library
//
// Programmers: Jamie Robertson, jamie@robertsonics.com
//
// **************************************************************
// modify by yves cordier yves.cordier@orange.fr
// **************************************************************
#include "wavTrigger.h"

#ifndef __WT_SERIAL_ASSIGNED__
wavTrigger::wavTrigger(HardwareSerial *camPort,unsigned long baud) // constructor
{
	
	comSpeed=9600;
	switch (baud)
	{
	case 1200 :{comSpeed=1200;break; }
	case 2400 :{comSpeed=2400;break; }
	case 4800 :{comSpeed=4800;break;}
	case 9600 :{comSpeed=9600;break; }
	case 19200 : {comSpeed=19200;break;}
	case 38400 : {comSpeed=38400;break;}
	case 57600 : {comSpeed=57600;break;}
		
		
	}
	_wavTrigger_Serial=camPort; // sets the serial port choses for the wave card (hardware ones)
	
}
#else
wavtTigger::wavTrigger() // constructor
{
	comSpeed=9600;
}
#endif

// **************************************************************
void wavTrigger::start( long speed) 
{
	uint8_t txbuf[5];

	versionRcvd = false;
	sysinfoRcvd = false;
	// new to commute automaticaly to the good serial 
	comSpeed=speed; 
	// 
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.begin(comSpeed);
	#else
	_wavTrigger_Serial->begin(comSpeed);
	#endif
	flush();
	

	// Request version string
	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x05;
	txbuf[3] = CMD_GET_VERSION;
	txbuf[4] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 5);
	#else
	_wavTrigger_Serial->write(txbuf, 5);
	#endif
	
	// Request system info
	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x05;
	txbuf[3] = CMD_GET_SYS_INFO;
	txbuf[4] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 5);
	#else
	_wavTrigger_Serial->write(txbuf, 5);
	#endif
	setReportingFlag=false; // at the begining 
}

// **************************************************************
void wavTrigger::flush(void) 
{
	int i;
	uint8_t dat;

	rxCount = 0;
	rxLen = 0;
	rxMsgReady = false;
	for (i = 0; i < MAX_NUM_VOICES; i++) {
		voiceTable[i] = 0xffff;
	}
	#ifdef __WT_SERIAL_ASSIGNED__ 
	while(WTSerial.available())
	dat = WTSerial.read();
	#else
	while(_wavTrigger_Serial->available())
	dat = _wavTrigger_Serial->read();
	#endif	
}


// **************************************************************
// modified by cordier yves 

void wavTrigger::update(void) 
{
	int i;
	uint8_t dat;
	uint8_t voice;
	uint16_t track;
	rxMsgReady = false;
	bool stay=true;
	int nbwaitloop=0;
	unsigned long tinit=millis();
	#ifdef __WT_SERIAL_ASSIGNED__ 
	while (stay ||WTSerial.available())
	#else
	while (stay || _wavTrigger_Serial->available())
	#endif	

	{
		#ifdef __WT_SERIAL_ASSIGNED__ 
		if(WTSerial.available()>0)
		
		#else
		if(_wavTrigger_Serial->available()>0)
		#endif		
		{   
			stay=true; 
			#ifdef __WT_SERIAL_ASSIGNED__ 
			dat = WTSerial.read();
			#else
			dat = _wavTrigger_Serial->read();
			#endif	
			switch (rxCount)
			{
			case 0 : 
				
				if (dat==SOM1) {rxCount++;}
				break;
				
			case 1 :
				
				if (dat == SOM2)
				{rxCount++;}
				else 
				{
					rxCount = 0;
					//Serial.print("Bad msg 1\n");
				}
				break;
				
			case 2 :
				
				if (dat <= MAX_MESSAGE_LEN) 
				{
					rxCount++;
					rxLen = dat - 1;
				}
				else 
				{
					rxCount = 0;
					//Serial.print("Bad msg 2\n");
				}
				break;
				
				default  :
				
				if (rxCount == rxLen) 
				{
					if (dat == EOM)
					rxMsgReady = true;
					else 
					{
						rxCount = 0;
						//Serial.print("Bad msg 3\n");
					}
				}
				else
				{
					if  (rxCount < rxLen) 
					{
						rxMessage[rxCount - 3] = dat; // save the message 
						rxCount++;
					}
					else  // rxCount>rxLen
					{
						rxCount = 0;
						//Serial.print("Bad msg 4\n");
					}
				}
				
			} // end of switch (rxcount)
			
			if (rxMsgReady) // we detetected a full message
			{
				switch (rxMessage[0]) // type of message ?
				{
				case RSP_TRACK_REPORT:
					track = rxMessage[2];
					track = (track << 8) + rxMessage[1] + 1;
					voice = rxMessage[3];
					if (voice < MAX_NUM_VOICES) {
						if (rxMessage[4] == 0) {
							if (track == voiceTable[voice])
							voiceTable[voice] = 0xffff;
						}
						else
						voiceTable[voice] = track;
					}
					// ==========================
					/*
					Serial.print("Track ");
					Serial.print(track);
					if (rxMessage[4] == 0)
						Serial.print(" off\n");
					else
						Serial.print(" on\n");
				*/
					// ==========================
					
					break;

				case RSP_VERSION_STRING:
					for (i = 0; i < (VERSION_STRING_LEN - 1); i++)
					version[i] = rxMessage[i + 1];
					version[VERSION_STRING_LEN - 1] = 0;
					versionRcvd = true;
					// ==========================
					/*
					Serial.write(version);
					Serial.write("\n");
					*/
					// ==========================
					
					break;

				case RSP_SYSTEM_INFO:
					numVoices = rxMessage[1];
					numTracks = rxMessage[3];
					numTracks = (numTracks << 8) + rxMessage[2];
					sysinfoRcvd = true;
					// ==========================
					///\Serial.print("Sys info received\n");
					// ==========================
					
					break;

				}
				rxCount = 0;
				rxLen = 0;
				rxMsgReady = false;
				stay=false;

			} // if (rxMsgReady)
			nbwaitloop=0; // we took a byte
		} // if char availabe from serial
		else
		{
			nbwaitloop++;
			if (nbwaitloop>20) 
			{stay=false;}
			else
			{
				int t=max(1,10000/comSpeed);
				delay(t); // wait for laps of time to get a byte from serial
			}
			
		} // if (_wavTrigger_Serial->available() )
	}// while (stay || _wavTrigger_Serial->available()
}

// **************************************************************

bool wavTrigger::isTrackPlaying(int trk) 


// simplifying Cordier Yves 20/02/17

{ 
	if (setReportingFlag) // in the opposit the response would be improbable
	{
		update(); // get informations sended
		for (int i = 0; i < MAX_NUM_VOICES; i++) 
		{
			if (voiceTable[i] == trk)
			{return true;} // exit in 'emergency' to avoid next analysis
		}
	}
	return false; // sorry not found
}

// **************************************************************
void wavTrigger::masterGain(int gain) 
{
	uint8_t txbuf[7];
	unsigned short vol;

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x07;
	txbuf[3] = CMD_MASTER_VOLUME;
	vol = (unsigned short)gain;
	txbuf[4] = (uint8_t)vol;
	txbuf[5] = (uint8_t)(vol >> 8);
	txbuf[6] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 7);
	#else
	_wavTrigger_Serial->write(txbuf, 7);
	#endif
}

// **************************************************************
void wavTrigger::setReporting(bool enable) 
{
	uint8_t txbuf[6];

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x06;
	txbuf[3] = CMD_SET_REPORTING;
	txbuf[4] = enable;
	txbuf[5] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 6);
	#else
	_wavTrigger_Serial->write(txbuf, 6);
	#endif
	setReportingFlag=true; // added 
}

// **************************************************************
bool wavTrigger::getVersion(char *pDst, int len) 
{
	int i;

	update();
	if (!versionRcvd) {
		return false;
	}
	for (i = 0; i < (VERSION_STRING_LEN - 1); i++) {
		if (i >= (len - 1))
		break;
		pDst[i] = version[i];
	}
	pDst[++i] = 0; // zero end string 
	return true;
}

// **************************************************************
int wavTrigger::getNumTracks(void) 
{
	update();
	return numTracks;
}

// **************************************************************
void wavTrigger::trackPlaySolo(int trk) 
{
	trackControl(trk, TRK_PLAY_SOLO);
}

// **************************************************************
void wavTrigger::trackPlaySolo(int trk, bool lock) 
{
	trackControl(trk, TRK_PLAY_SOLO, lock);
}

// **************************************************************

void wavTrigger::trackPlayPoly(int trk) 
{
	trackControl(trk, TRK_PLAY_POLY);
}

// **************************************************************
void wavTrigger::trackPlayPoly(int trk, bool lock) 
{
	trackControl(trk, TRK_PLAY_POLY, lock);
}

// **************************************************************

void wavTrigger::trackLoad(int trk) 
{
	trackControl(trk, TRK_LOAD);
}

// **************************************************************
void wavTrigger::trackLoad(int trk, bool lock) 
{
	trackControl(trk, TRK_LOAD, lock);
}

// **************************************************************
void wavTrigger::trackStop(int trk) 
{
	trackControl(trk, TRK_STOP);
}

// **************************************************************
void wavTrigger::trackPause(int trk) 
{
	trackControl(trk, TRK_PAUSE);
}

// **************************************************************
void wavTrigger::trackResume(int trk) 
{
	trackControl(trk, TRK_RESUME);
}

// **************************************************************
void wavTrigger::trackLoop(int trk, bool enable) 
{
	if (enable)
	trackControl(trk, TRK_LOOP_ON);
	else
	trackControl(trk, TRK_LOOP_OFF);
}


// **************************************************************

void wavTrigger::trackControl(int trk, int code) 
{

	uint8_t txbuf[8];

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x08;
	txbuf[3] = CMD_TRACK_CONTROL;
	txbuf[4] = (uint8_t)code;
	txbuf[5] = (uint8_t)trk;
	txbuf[6] = (uint8_t)(trk >> 8);
	txbuf[7] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 8);
	#else
	_wavTrigger_Serial->write(txbuf, 8);
	#endif
}



// **************************************************************
void wavTrigger::trackControl(int trk, int code, bool lock ) 

{
	uint8_t txbuf[9];

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x09;
	txbuf[3] = CMD_TRACK_CONTROL_EX;
	txbuf[4] = (uint8_t)code;
	txbuf[5] = (uint8_t)trk;
	txbuf[6] = (uint8_t)(trk >> 8);
	txbuf[7] = lock;
	txbuf[8] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 9);
	#else
	_wavTrigger_Serial->write(txbuf, 9);
	#endif
	
}

// **************************************************************
void wavTrigger::stopAllTracks(void) 
{
	uint8_t txbuf[5];

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x05;
	txbuf[3] = CMD_STOP_ALL;
	txbuf[4] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 5);
	#else
	_wavTrigger_Serial->write(txbuf, 5);
	#endif
}

// **************************************************************
void wavTrigger::resumeAllInSync(void) 
{
	uint8_t txbuf[5];

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x05;
	txbuf[3] = CMD_RESUME_ALL_SYNC;
	txbuf[4] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 5);
	#else
	_wavTrigger_Serial->write(txbuf, 5);
	#endif
	
}

// **************************************************************
void wavTrigger::trackGain(int trk, int gain) 
{
	uint8_t txbuf[9];
	unsigned short vol;

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x09;
	txbuf[3] = CMD_TRACK_VOLUME;
	txbuf[4] = (uint8_t)trk;
	txbuf[5] = (uint8_t)(trk >> 8);
	vol = (unsigned short)gain;
	txbuf[6] = (uint8_t)vol;
	txbuf[7] = (uint8_t)(vol >> 8);
	txbuf[8] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 9);
	#else
	_wavTrigger_Serial->write(txbuf, 9);
	#endif
}

// **************************************************************
void wavTrigger::trackFade(int trk, int gain, int time, bool stopFlag) 
{
	uint8_t txbuf[12];
	unsigned short vol;

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x0c;
	txbuf[3] = CMD_TRACK_FADE;
	txbuf[4] = (uint8_t)trk;
	txbuf[5] = (uint8_t)(trk >> 8);
	vol = (unsigned short)gain;
	txbuf[6] = (uint8_t)vol;
	txbuf[7] = (uint8_t)(vol >> 8);
	txbuf[8] = (uint8_t)time;
	txbuf[9] = (uint8_t)(time >> 8);
	txbuf[10] = stopFlag;
	txbuf[11] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 12);
	#else
	_wavTrigger_Serial->write(txbuf, 12);
	#endif
}

// **************************************************************
void wavTrigger::samplerateOffset(int offset) 
{
	uint8_t txbuf[7];
	unsigned short off;

	txbuf[0] = SOM1;
	txbuf[1] = SOM2;
	txbuf[2] = 0x07;
	txbuf[3] = CMD_SAMPLERATE_OFFSET;
	off = (unsigned short)offset;
	txbuf[4] = (uint8_t)off;
	txbuf[5] = (uint8_t)(off >> 8);
	txbuf[6] = EOM;
	#ifdef __WT_SERIAL_ASSIGNED__
	WTSerial.write(txbuf, 7);
	#else
	_wavTrigger_Serial->write(txbuf, 7);
	#endif
	
}

// **************************************************************
// added by Cordier Yves

long wavTrigger::getComSpeed() {return comSpeed;}

// **************************************************************


